#! /usr/local/bin/perl

require 5.004;

use SHA;
use MD5;

# Implement RFC 2104 using SHA-1.
# Parameters: secret key, string to encode [, test flag]
# If the test flag is present and nonzero, we return the hash
# in hexadecimal, otherwise in raw binary.
sub HMAC_SHA1 {
    my ($SHA, $B, $L, $ipad, $opad, $K, $data, @step, $RFCTEST);

    $SHA = new SHA;
    $B = 64;
    $L = 20;
    $ipad = chr(0x36) x $B;
    $opad = chr(0x5C) x $B;

    $K = shift;
    $data = shift;
    $RFCTEST = shift;

    if (length($K) > $B) { $K = $SHA->hash($K) }

    $step[2] = ($K ^ $ipad);
    $step[3] = $step[2] . $data;
    $step[4] = $SHA->hash($step[3]);
    $step[5] = ($K ^ $opad);
    $step[6] = $step[5] . $step[4];

    if ($RFCTEST) { return $SHA->hexhash($step[6]) }
    else { return $SHA->hash($step[6]) }

}

# Implement RFC 2104 using MD5.
# Parameters: secret key, string to encode [, test flag]
# If the test flag is present and nonzero, we return the hash
# in hexadecimal, otherwise in raw binary.
sub HMAC_MD5 {
    my ($MD5, $B, $L, $ipad, $opad, $K, $data, @step, $RFCTEST);

    $MD5 = new MD5;
    $B = 64;
    $L = 20;
    $ipad = chr(0x36) x $B;
    $opad = chr(0x5C) x $B;

    $K = shift;
    $data = shift;
    $RFCTEST = shift;

    if (length($K) > $B) { $K = $MD5->hash($K) }

    $step[2] = ($K ^ $ipad);
    $step[3] = $step[2] . $data;
    $step[4] = $MD5->hash($step[3]);
    $step[5] = ($K ^ $opad);
    $step[6] = $step[5] . $step[4];

    if ($RFCTEST) { return $MD5->hexhash($step[6]) }
    else { return $MD5->hash($step[6]) }

}

sub unhex {
  my $string = shift;
  my $i;
  for ($i = 0; $i < length($string); $i++) {
    printf("%02x", ord(substr($string, $i, 1)));
  }
  print "\n";
}
